VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CPolygonMask"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' *********************************************************************
'  Copyright 1998 Karl E. Peterson, All Rights Reserved
'  http://www.mvps.org/vb
' *********************************************************************
'  You are free to use this code within your own applications, but you
'  are expressly forbidden from selling or otherwise distributing this
'  source code without prior written consent.
' *********************************************************************
Option Explicit

Private Declare Function GetDesktopWindow Lib "user32" () As Long
Private Declare Function GetDC Lib "user32" (ByVal hwnd As Long) As Long
Private Declare Function CreateCompatibleDC Lib "gdi32" (ByVal hDC As Long) As Long
Private Declare Function ReleaseDC Lib "user32" (ByVal hwnd As Long, ByVal hDC As Long) As Long
Private Declare Function GetObj Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hDC As Long, ByVal hObject As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long
Private Declare Function CreateBitmap Lib "gdi32" (ByVal nWidth As Long, ByVal nHeight As Long, ByVal nPlanes As Long, ByVal nBitCount As Long, lpBits As Any) As Long
Private Declare Function FillRect Lib "user32" (ByVal hDC As Long, lpRect As RECT, ByVal hBrush As Long) As Long
Private Declare Function DeleteDC Lib "gdi32" (ByVal hDC As Long) As Long
Private Declare Function GetStockObject Lib "gdi32" (ByVal nIndex As Long) As Long
Private Declare Sub CopyMem Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)
Private Declare Function Polygon Lib "gdi32" (ByVal hDC As Long, lpPoint As POINTAPI, ByVal nCount As Long) As Long
Private Declare Function OleCreatePictureIndirect Lib "olepro32.dll" (PicDesc As PicBmp, RefIID As GUID, ByVal fPictureOwnsHandle As Long, IPic As IPicture) As Long
Private Declare Function BitBlt Lib "gdi32" (ByVal hDestDC As Long, ByVal X As Long, ByVal Y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hSrcDC As Long, ByVal xSrc As Long, ByVal ySrc As Long, ByVal dwRop As Long) As Long

Private Type RECT
   Left As Long
   Top As Long
   Right As Long
   Bottom As Long
End Type

Private Type POINTAPI
   X As Long
   Y As Long
End Type

Private Type PicBmp
   Size As Long
   Type As Long
   hBmp As Long
   hPal As Long
   Reserved As Long
End Type

Private Type GUID
   Data1 As Long
   Data2 As Integer
   Data3 As Integer
   Data4(7) As Byte
End Type

Private Const WHITE_BRUSH = 0
Private Const BLACK_BRUSH = 4
Private Const BLACK_PEN = 7

Private m_Parent As Long
Private m_Width As Long
Private m_Height As Long
Private m_Sides As Long
Private m_Offset As Long
Private m_Pic As IPicture
Private m_Pts() As POINTAPI

' ****************************************************
'  Initialization and Termination
' ****************************************************
Private Sub Class_Initialize()
   '
End Sub

Private Sub Class_Terminate()
   '
End Sub

' ****************************************************
'  Public Properties
' ****************************************************
Public Property Let Parent(ByVal NewVal As Long)
   m_Parent = NewVal
End Property

Public Property Get Parent() As Long
   Parent = m_Parent
End Property

' ****************************************************
'  Public Properties (Read-Only)
' ****************************************************
Public Property Get Mask() As IPicture
   Set Mask = m_Pic
End Property

' ****************************************************
'  Public Methods
' ****************************************************
Public Sub RenderMask()
   If ReadProperties Then
      Call PreparePoints
      Call CreateMask
   End If
End Sub

' ****************************************************
'  Private Methods
' ****************************************************
Private Function ReadProperties() As Boolean
   Dim obj As PolyButton
   If m_Parent Then
      Call CopyMem(obj, m_Parent, 4)
      m_Sides = obj.Sides
      m_Offset = obj.Offset
      m_Width = obj.ScaleWidth
      m_Height = obj.ScaleHeight
      Call CopyMem(obj, 0&, 4)
      ReadProperties = True
   End If
End Function

Private Sub PreparePoints()
   Dim Angle As Double
   Dim Slice As Double
   Dim X1 As Long, Y1 As Long
   Dim radius As Long
   Dim theta As Double
   Dim n As Long, i As Long
   '
   ' Some useful constants.
   '
   Const Pi# = 3.14159265358979
   Const DegToRad# = Pi / 180
   '
   ' Calc angle between each point, centerpoint, and radius.
   '
   Slice = 360 / m_Sides
   X1 = m_Width \ 2
   Y1 = m_Height \ 2
   If X1 > Y1 Then
      radius = Y1 - 1
   Else
      radius = X1 - 1
   End If
   '
   ' Calculate endpoints for each vertex
   '
   ReDim m_Pts(0 To m_Sides - 1) As POINTAPI
   Angle = 180# - m_Offset
   For i = 0 To m_Sides - 1
      theta = Angle * DegToRad
      m_Pts(i).X = X1 + (radius * (Sin(theta)))
      m_Pts(i).Y = Y1 + (radius * (Cos(theta)))
      Angle = Angle + Slice
   Next i
End Sub

Private Sub CreateMask()
   Dim hWndScn As Long
   Dim hDCScn As Long
   Dim hDC As Long
   Dim hBmp As Long       ' Bitmap stores mask (monochrome)
   Dim hBmpPrev As Long
   Dim hPenPrev As Long
   Dim hBrushPrev As Long
   Dim rMask As RECT
   '
   ' Get desktop DC, and create compatable DCs.
   '
   hWndScn = GetDesktopWindow()
   hDCScn = GetDC(hWndScn)
   hDC = CreateCompatibleDC(hDCScn)
   Call ReleaseDC(hWndScn, hDCScn)
   '
   ' Create mono BMP
   '
   hBmp = CreateBitmap(m_Width, m_Height, 1, 1, ByVal 0&)
   '
   ' Select BMP into DC, storing previous bitmap.
   '
   hBmpPrev = SelectObject(hDC, hBmp)
   '
   ' Flood bitmap with white.
   '
   rMask.Right = m_Width
   rMask.Bottom = m_Height
   Call FillRect(hDC, rMask, GetStockObject(WHITE_BRUSH))
   '
   ' Draw polygon in black
   '
   hPenPrev = SelectObject(hDC, GetStockObject(BLACK_PEN))
   hBrushPrev = SelectObject(hDC, GetStockObject(BLACK_BRUSH))
   Call Polygon(hDC, m_Pts(0), m_Sides)
   Call SelectObject(hDC, hPenPrev)
   Call SelectObject(hDC, hBrushPrev)
   '
   ' Remove the new copy of the bitmap.
   '
   hBmp = SelectObject(hDC, hBmpPrev)
   '
   ' Create a picture object from memory bitmap.
   '
   Call CreateBitmapPicture(hBmp)
   '
   ' Clean up
   '
   Call DeleteDC(hDC)
End Sub

Private Sub CreateBitmapPicture(ByVal hBmp As Long)
   Dim pic As PicBmp
   Dim IPic As IPicture
   Dim IID_IDispatch As GUID
   '
   ' Fill in with IDispatch Interface ID
   '
   With IID_IDispatch
      .Data1 = &H20400
      .Data4(0) = &HC0
      .Data4(7) = &H46
   End With
   '
   ' Fill PicBmp struct with necessary parts
   '
   With pic
      .Size = Len(pic)           ' Length of structure
      .Type = vbPicTypeBitmap    ' Type of Picture
      .hBmp = hBmp               ' Handle to bitmap
      .hPal = 0                  ' Handle to palette (may be null)
   End With
   '
   ' Clear old instance of picture object.
   '
   Set m_Pic = Nothing
   '
   ' Create Picture object
   '
   OleCreatePictureIndirect pic, IID_IDispatch, 1, m_Pic
End Sub
