VERSION 5.00
Begin VB.UserControl PolyButton 
   ClientHeight    =   2445
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   3450
   ScaleHeight     =   163
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   230
   ToolboxBitmap   =   "PolyButton.ctx":0000
End
Attribute VB_Name = "PolyButton"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' *********************************************************************
'  Copyright 1998 Karl E. Peterson, All Rights Reserved
'  http://www.mvps.org/vb
' *********************************************************************
'  You are free to use this code within your own applications, but you
'  are expressly forbidden from selling or otherwise distributing this
'  source code without prior written consent.
' *********************************************************************
Option Explicit

Private Declare Function Polygon Lib "gdi32" (ByVal hDC As Long, lpPoint As POINTAPI, ByVal nCount As Long) As Long
Private Declare Function GetSysColor Lib "user32" (ByVal nIndex As Long) As Long

Private Type POINTAPI
   X As Long
   Y As Long
End Type

Private Const defBackColor = vbButtonFace
Private Const defForeColor = vbWindowText
Private Const defOffset = 0
Private Const defSides = 5

Private m_BackColor As OLE_COLOR
Private m_Caption As String
Private m_Font As Font
Private m_ForeColor As OLE_COLOR
Private m_Offset As Double
Private m_Sides As Long

Private m_Pts() As POINTAPI
Private m_Mask As CPolygonMask

Event MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
Attribute MouseDown.VB_UserMemId = -605
Event MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
Attribute MouseMove.VB_UserMemId = -606
Event MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
Attribute MouseUp.VB_UserMemId = -607

' ****************************************************
'  Initialization and Termination
' ****************************************************
Private Sub UserControl_Initialize()
   Set m_Mask = New CPolygonMask
   m_Mask.Parent = ObjPtr(Me)
End Sub

Private Sub UserControl_InitProperties()
   m_BackColor = defBackColor
   m_Caption = Extender.Name
   Set m_Font = Ambient.Font
   m_ForeColor = defForeColor
   m_Offset = defOffset
   m_Sides = defSides
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
   With PropBag
      m_BackColor = .ReadProperty("BackColor", defBackColor)
      m_Caption = .ReadProperty("Caption", Extender.Name)
      Set m_Font = .ReadProperty("Font", Ambient.Font)
      m_ForeColor = .ReadProperty("ForeColor", defForeColor)
      m_Offset = .ReadProperty("Offset", defOffset)
      m_Sides = .ReadProperty("Sides", defSides)
   End With
   
   With UserControl
      .AutoRedraw = True
      .BackStyle = vbTransparent
      Set .Font = m_Font
      .MaskColor = vbWhite
   End With
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
   With PropBag
      .WriteProperty "BackColor", m_BackColor, defBackColor
      .WriteProperty "Caption", m_Caption, Extender.Name
      .WriteProperty "Font", m_Font, Ambient.Font
      .WriteProperty "ForeColor", m_ForeColor, defForeColor
      .WriteProperty "Offset", m_Offset, defOffset
      .WriteProperty "Sides", m_Sides, defSides
   End With
End Sub

' ****************************************************
'  UserControl Events
' ****************************************************
Private Sub UserControl_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
   If Button And vbLeftButton Then
      m_BackColor = InverseRGB(m_BackColor)
      Me.Refresh
   End If
   
   RaiseEvent MouseDown(Button, Shift, _
      ScaleX(X, vbPixels, vbContainerSize), _
      ScaleY(Y, vbPixels, vbContainerSize))
End Sub

Private Sub UserControl_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
   RaiseEvent MouseMove(Button, Shift, _
      ScaleX(X, vbPixels, vbContainerSize), _
      ScaleY(Y, vbPixels, vbContainerSize))
End Sub

Private Sub UserControl_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
   If Button And vbLeftButton Then
      m_BackColor = InverseRGB(m_BackColor)
      Me.Refresh
   End If
   
   RaiseEvent MouseUp(Button, Shift, _
      ScaleX(X, vbPixels, vbContainerSize), _
      ScaleY(Y, vbPixels, vbContainerSize))
End Sub

Private Sub UserControl_Paint()
   Call Me.Refresh
End Sub

Private Sub UserControl_Resize()
   Call Me.Refresh
End Sub

' ****************************************************
'  Public Properties
' ****************************************************
Public Property Let BackColor(ByVal NewVal As OLE_COLOR)
   m_BackColor = NewVal
   PropertyChanged "BackColor"
   Me.Refresh
End Property

Public Property Get BackColor() As OLE_COLOR
   BackColor = m_BackColor
End Property

Public Property Let Caption(ByVal NewVal As String)
   m_Caption = NewVal
   PropertyChanged "Caption"
   Me.Refresh
End Property

Public Property Get Caption() As String
Attribute Caption.VB_UserMemId = -518
   Caption = m_Caption
End Property

Public Property Set Font(ByVal NewFont As Font)
   If Not (NewFont Is Nothing) Then
      Set m_Font = NewFont
      Set UserControl.Font = m_Font
      PropertyChanged "Font"
      Me.Refresh
   End If
End Property

Public Property Get Font() As Font
Attribute Font.VB_UserMemId = -512
   Set Font = m_Font
End Property

Public Property Let ForeColor(ByVal NewVal As OLE_COLOR)
   m_ForeColor = NewVal
   PropertyChanged "ForeColor"
   Me.Refresh
End Property

Public Property Get ForeColor() As OLE_COLOR
Attribute ForeColor.VB_UserMemId = -513
   ForeColor = m_ForeColor
End Property

Public Property Let Offset(ByVal NewVal As Double)
   m_Offset = NewVal
   PropertyChanged "Offset"
   Me.Refresh
End Property

Public Property Get Offset() As Double
   Offset = m_Offset
End Property

Public Property Let Sides(ByVal NewVal As Long)
   If NewVal > 1 Then
      m_Sides = NewVal
      PropertyChanged "Sides"
      Me.Refresh
   End If
End Property

Public Property Get Sides() As Long
   Sides = m_Sides
End Property

' ****************************************************
'  Public Properties (Read-Only)
' ****************************************************
Public Property Get ScaleHeight() As Long
   ScaleHeight = UserControl.ScaleHeight
End Property

Public Property Get ScaleWidth() As Long
   ScaleWidth = UserControl.ScaleWidth
End Property

' ****************************************************
'  Public Methods
' ****************************************************
Public Sub Refresh()
   Dim p As Picture
   With UserControl
      .Cls
      .BackColor = m_BackColor
      .ForeColor = m_ForeColor
      Call m_Mask.RenderMask
      .MaskPicture = m_Mask.Mask
      Call PreparePoints(.ScaleWidth, .ScaleHeight)
      Call Polygon(.hDC, m_Pts(0), m_Sides)
      .CurrentX = (.ScaleWidth - .TextWidth(m_Caption)) \ 2
      .CurrentY = (.ScaleHeight - .TextHeight(m_Caption)) \ 2
      UserControl.Print m_Caption
   End With
End Sub

' ****************************************************
'  Private Methods
' ****************************************************
Private Sub PreparePoints(ByVal nWidth As Long, nHeight As Long)
   Dim Angle As Double
   Dim Slice As Double
   Dim X1 As Long, Y1 As Long
   Dim radius As Long
   Dim theta As Double
   Dim n As Long, i As Long
   '
   ' Some useful constants.
   '
   Const Pi# = 3.14159265358979
   Const DegToRad# = Pi / 180
   '
   ' Calc angle between each point, centerpoint, and radius.
   '
   Slice = 360 / m_Sides
   X1 = nWidth \ 2
   Y1 = nHeight \ 2
   If X1 > Y1 Then
      radius = Y1 - 1
   Else
      radius = X1 - 1
   End If
   '
   ' Calculate endpoints for each vertex
   '
   ReDim m_Pts(0 To m_Sides - 1) As POINTAPI
   Angle = 180# - m_Offset
   For i = 0 To m_Sides - 1
      theta = Angle * DegToRad
      m_Pts(i).X = X1 + (radius * (Sin(theta)))
      m_Pts(i).Y = Y1 + (radius * (Cos(theta)))
      Angle = Angle + Slice
   Next i
End Sub

Private Function InverseRGB(ByVal ColorRef As OLE_COLOR) As Long
   Dim red As Integer
   Dim green As Integer
   Dim blue As Integer
   Const HighBit = &H80000000
   '
   ' Get system defined RGB value, if an OLE color.
   '
   If ColorRef And HighBit Then
      ColorRef = GetSysColor(ColorRef And Not HighBit)
   End If
   '
   ' Break out RGB values.
   '
   red = (ColorRef Mod &H100)
   green = (ColorRef \ &H100) Mod &H100
   blue = (ColorRef \ &H10000) Mod &H10000
   '
   ' Pop out the inverse.
   '
   InverseRGB = RGB(255 - red, 255 - green, 255 - blue)
End Function
